@@ -124,9 +124,6 @@ def check_args_rule(command: Command) -> T_RuleChecker:
             return False
 
         sender = UserInfo(qq=str(event.user_id))
-        await get_user_info(bot, sender)
-        for user in users:
-            await get_user_info(bot, user)
         state["sender"] = sender
         state["users"] = users
         state["args"] = args
@@ -162,10 +159,14 @@ async def handle(
 
 def create_matchers():
     def create_handler(command: Command) -> T_Handler:
-        async def handler(state: T_State = State()):
-            await handle(
-                matcher, command, state["sender"], state["users"], state["args"]
-            )
+        async def handler(bot: Bot, state: T_State = State()):
+            sender: UserInfo = state["sender"]
+            users: List[UserInfo] = state["users"]
+            args: List[str] = state["args"]
+            await get_user_info(bot, sender)
+            for user in users:
+                await get_user_info(bot, user)
+            await handle(matcher, command, sender, users, args)
 
         return handler
 
