@@ -39,10 +39,20 @@ commands = [
     Command(("继续干活",), back_to_work, convert=False),
     Command(("完美", "完美的"), perfect, convert=False),
     Command(("关注",), follow, arg_num=1),
-    Command(("我朋友说", "我有个朋友说"), my_friend, arg_num=2),
+    Command(("我朋友说", "我有个朋友说"), my_friend, arg_num=10),
     Command(("这像画吗",), paint, convert=False),
     Command(("震惊",), shock),
-    Command(("兑换券",), coupon, arg_num=1),
+    Command(("兑换券",), coupon, arg_num=2),
+    Command(("听音乐",), listen_music),
+    Command(("典中典",), dianzhongdian, convert=False, arg_num=3),
+    Command(("哈哈镜",), funny_mirror),
+    Command(("永远爱你",), love_you),
+    Command(("对称",), symmetric, convert=False, arg_num=1),
+    Command(("安全感",), safe_sense, convert=False, arg_num=2),
+    Command(("永远喜欢", "我永远喜欢"), always_like, convert=False, arg_num=10),
+    Command(("采访",), interview, arg_num=1),
+    Command(("打拳",), punch, convert=False),
+    Command(("群青",), cyan),
 ]
 
 
