@@ -1,18 +1,25 @@
 import math
+import httpx
 import imageio
 import cv2 as cv
 import numpy as np
 from enum import Enum
 from io import BytesIO
-from typing import Callable, List, Tuple, Union
+from fontTools.ttLib import TTFont
+from typing import Protocol, List, Tuple, Union
+from typing_extensions import Literal
 from PIL.Image import Image as IMG
 from PIL.ImageFont import FreeTypeFont
 from PIL import Image, ImageDraw, ImageFilter, ImageFont
+from emoji.unicode_codes import UNICODE_EMOJI
+from nonebot.log import logger
 
 from .download import get_font, get_image
 from .models import Command
 
 DEFAULT_FONT = "SourceHanSansSC-Regular.otf"
+BOLD_FONT = "SourceHanSansSC-Bold.otf"
+EMOJI_FONT = "NotoColorEmoji.ttf"
 
 
 def resize(img: IMG, size: Tuple[int, int]) -> IMG:
@@ -33,16 +40,71 @@ def circle(img: IMG) -> IMG:
 
 
 def square(img: IMG) -> IMG:
-    width, height = img.size
-    length = min(width, height)
-    return img.crop(
-        (
-            int((width - length) / 2),
-            int((height - length) / 2),
-            int((width + length) / 2),
-            int((height + length) / 2),
-        )
-    )
+    length = min(img.width, img.height)
+    return cut_size(img, (length, length))
+
+
+async def draw_text(
+    img: IMG,
+    pos: Tuple[float, float],
+    text: str,
+    font: FreeTypeFont,
+    fill=None,
+    spacing: int = 4,
+    align: Literal["left", "right", "center"] = "left",
+    stroke_width: int = 0,
+    stroke_fill=None,
+):
+    if not text:
+        return
+    lines = text.strip().split("\n")
+    logger.info(lines)
+    draw = ImageDraw.Draw(img)
+    max_w = font.getsize_multiline(text, spacing=spacing, stroke_width=stroke_width)[0]
+    current_x, current_y = pos
+
+    emoji_font_file = BytesIO(await get_font(EMOJI_FONT))
+    emoji_font = ImageFont.truetype(emoji_font_file, 109, encoding="utf-8")
+    emoji_ttf = TTFont(emoji_font_file)
+
+    def has_emoji(emoji: str):
+        for table in emoji_ttf["cmap"].tables:  # type: ignore
+            if ord(emoji) in table.cmap.keys():
+                return True
+        return False
+
+    for line in lines:
+        line_w = font.getsize(line, stroke_width=stroke_width)[0]
+        dw = max_w - line_w
+        current_x = pos[0]
+        if align == "center":
+            current_x += dw / 2
+        elif align == "right":
+            current_x += dw
+
+        for char in line:
+            if char in UNICODE_EMOJI["en"] and has_emoji(char):
+                emoji_img = Image.new("RGBA", (150, 150))
+                emoji_draw = ImageDraw.Draw(emoji_img)
+                emoji_draw.text((0, 0), char, font=emoji_font, embedded_color=True)
+                emoji_img = emoji_img.crop(emoji_font.getbbox(char))
+                emoji_x, emoji_y = font.getsize(char, stroke_width=stroke_width)
+                emoji_img = fit_size(
+                    emoji_img, (emoji_x, emoji_y), FitSizeMode.INSIDE, FitSizeDir.SOUTH
+                )
+                img.paste(emoji_img, (int(current_x), int(current_y)), emoji_img)
+                current_x += emoji_x
+            else:
+                draw.text(
+                    (current_x, current_y),
+                    char,
+                    font=font,
+                    fill=fill,
+                    stroke_width=stroke_width,
+                    stroke_fill=stroke_fill,
+                )
+                current_x += font.getsize(char, stroke_width=stroke_width)[0]
+        current_y += font.getsize("A", stroke_width=stroke_width)[1] + spacing
 
 
 # 适应大小模式
@@ -64,21 +126,15 @@ class FitSizeDir(Enum):
     SOUTHEAST = 8
 
 
-def fit_size(
-    img: IMG,
-    size: Tuple[int, int],
-    mode: FitSizeMode = FitSizeMode.INCLUDE,
-    direction: FitSizeDir = FitSizeDir.CENTER,
-    bg_color: Union[str, float, Tuple[float, ...]] = (255, 255, 255, 0),
+def limit_size(
+    img: IMG, size: Tuple[int, int], mode: FitSizeMode = FitSizeMode.INCLUDE
 ) -> IMG:
     """
-    调整图片到指定的大小
+    调整图片到指定的大小，不改变长宽比（即返回的图片大小不一定是指定的size）
     :params
       * ``img``: 待调整的图片
       * ``size``: 期望图片大小
-      * ``mode``: FitSizeMode.INSIDE 表示图片必须在指定的大小范围内，不足部分留空；FitSizeMode.INCLUDE 表示图片必须包括指定的大小范围，超出部分裁剪
-      * ``direction``: 调整图片大小时图片的方位；默认为居中 FitSizeDir.CENTER
-      * ``bg_color``: FitSizeMode.INSIDE 时的背景颜色
+      * ``mode``: FitSizeMode.INSIDE 表示图片必须在指定的大小范围内；FitSizeMode.INCLUDE 表示图片必须包括指定的大小范围
     """
     w, h = size
     img_w, img_h = img.size
@@ -88,8 +144,25 @@ def fit_size(
         ratio = max(w / img_w, h / img_h)
     img_w = int(img_w * ratio)
     img_h = int(img_h * ratio)
-    img = resize(img, (img_w, img_h))
+    return resize(img, (img_w, img_h))
 
+
+def cut_size(
+    img: IMG,
+    size: Tuple[int, int],
+    direction: FitSizeDir = FitSizeDir.CENTER,
+    bg_color: Union[str, float, Tuple[float, ...]] = (255, 255, 255, 0),
+) -> IMG:
+    """
+    裁剪图片到指定的大小，超出部分裁剪，不足部分设为指定颜色
+    :params
+      * ``img``: 待调整的图片
+      * ``size``: 期望图片大小
+      * ``direction``: 调整图片大小时图片的方位；默认为居中 FitSizeDir.CENTER
+      * ``bg_color``: FitSizeMode.INSIDE 时的背景颜色
+    """
+    w, h = size
+    img_w, img_h = img.size
     x = int((w - img_w) / 2)
     y = int((h - img_h) / 2)
     if direction in [FitSizeDir.NORTH, FitSizeDir.NORTHWEST, FitSizeDir.NORTHEAST]:
@@ -105,6 +178,25 @@ def fit_size(
     return result
 
 
+def fit_size(
+    img: IMG,
+    size: Tuple[int, int],
+    mode: FitSizeMode = FitSizeMode.INCLUDE,
+    direction: FitSizeDir = FitSizeDir.CENTER,
+    bg_color: Union[str, float, Tuple[float, ...]] = (255, 255, 255, 0),
+) -> IMG:
+    """
+    调整图片到指定的大小，超出部分裁剪，不足部分设为指定颜色
+    :params
+      * ``img``: 待调整的图片
+      * ``size``: 期望图片大小
+      * ``mode``: FitSizeMode.INSIDE 表示图片必须在指定的大小范围内，不足部分设为指定颜色；FitSizeMode.INCLUDE 表示图片必须包括指定的大小范围，超出部分裁剪
+      * ``direction``: 调整图片大小时图片的方位；默认为居中 FitSizeDir.CENTER
+      * ``bg_color``: FitSizeMode.INSIDE 时的背景颜色
+    """
+    return cut_size(limit_size(img, size, mode), size, direction, bg_color)
+
+
 def perspective(img: IMG, points: List[Tuple[float, float]]):
     """
     透视变换
@@ -144,6 +236,39 @@ def motion_blur(img: IMG, angle=0, degree=0) -> IMG:
     return Image.fromarray(np.array(blurred, dtype=np.uint8))
 
 
+def distort(img: IMG, coefficients: Tuple[float, float, float, float]) -> IMG:
+    res = cv.undistort(
+        np.asarray(img),
+        np.array([[100, 0, img.width / 2], [0, 100, img.height / 2], [0, 0, 1]]),
+        np.asarray(coefficients),
+    )
+    return Image.fromarray(np.array(res, dtype=np.uint8))
+
+
+def color_mask(img: IMG, color: Tuple[int, int, int]) -> IMG:
+    img = img.convert("RGB")
+    w, h = img.size
+    img_array = np.asarray(img)
+    img_gray = cv.cvtColor(img_array, cv.COLOR_RGB2GRAY)
+    img_hsl = cv.cvtColor(img_array, cv.COLOR_RGB2HLS)
+    img_new = np.zeros((h, w, 3), np.uint8)
+    r, g, b = color
+    rgb_sum = sum(color)
+    for i in range(h):
+        for j in range(w):
+            value = img_gray[i, j]
+            new_color = [
+                int(value * r / rgb_sum),
+                int(value * g / rgb_sum),
+                int(value * b / rgb_sum),
+            ]
+            img_new[i, j] = new_color
+    img_new_hsl = cv.cvtColor(img_new, cv.COLOR_RGB2HLS)
+    result = np.dstack((img_new_hsl[:, :, 0], img_hsl[:, :, 1], img_new_hsl[:, :, 2]))
+    result = cv.cvtColor(result, cv.COLOR_HLS2RGB)
+    return Image.fromarray(result)
+
+
 def save_gif(frames: List[IMG], duration: float) -> BytesIO:
     output = BytesIO()
     imageio.mimsave(output, frames, format="gif", duration=duration)
@@ -166,8 +291,13 @@ def save_jpg(frame: IMG) -> BytesIO:
     return output
 
 
-def make_jpg_or_gif(
-    img: IMG, func: Callable[[IMG], IMG], gif_zoom: float = 1, gif_max_frames: int = 50
+class Maker(Protocol):
+    async def __call__(self, img: IMG) -> IMG:
+        ...
+
+
+async def make_jpg_or_gif(
+    img: IMG, func: Maker, gif_zoom: float = 1, gif_max_frames: int = 50
 ) -> BytesIO:
     """
     制作静图或者动图
@@ -178,7 +308,7 @@ def make_jpg_or_gif(
       * ``direction``: gif 最大帧数，避免生成的 gif 太大
     """
     if not getattr(img, "is_animated", False):
-        return save_jpg(func(img))
+        return save_jpg(await func(img))
     else:
         index = range(img.n_frames)
         ratio = img.n_frames / gif_max_frames
@@ -190,7 +320,7 @@ def make_jpg_or_gif(
         frames = []
         for i in index:
             img.seek(i)
-            new_img = func(img)
+            new_img = await func(img)
             frames.append(
                 resize(
                     new_img,
@@ -217,6 +347,22 @@ async def load_font(name: str, fontsize: int) -> FreeTypeFont:
     return ImageFont.truetype(BytesIO(font), fontsize, encoding="utf-8")
 
 
+def wrap_text(text: str, font: FreeTypeFont, max_width: float, **kwargs) -> List[str]:
+    line = ""
+    lines = []
+    for t in text:
+        if t == "\n":
+            lines.append(line)
+            line = ""
+        elif font.getsize(line + t, **kwargs)[0] > max_width:
+            lines.append(line)
+            line = t
+        else:
+            line += t
+    lines.append(line)
+    return lines
+
+
 async def fit_font_size(
     text: str,
     max_width: float,
@@ -240,6 +386,18 @@ async def fit_font_size(
             return 0
 
 
+async def translate(text: str) -> str:
+    url = f"http://fanyi.youdao.com/translate"
+    params = {"type": "ZH_CN2JA", "i": text, "doctype": "json"}
+    try:
+        async with httpx.AsyncClient() as client:
+            resp = await client.get(url, params=params)
+            result = resp.json()
+        return result["translateResult"][0][0]["tgt"]
+    except:
+        return ""
+
+
 async def help_image(commands: List[Command]) -> BytesIO:
     font = await load_font(DEFAULT_FONT, 30)
     padding = 10
